package com.mh.user.aspect;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.mh.user.annotation.FieldParam;
import com.mh.user.annotation.TranslationDict;
import com.mh.user.model.SysDict;
import com.mh.user.service.SysDictService;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 数据字典转换
 * @date 2024-11-08 14:08:20
 */
@Aspect
@Component
@Slf4j
public class DictDataTranslateAsp {

    @Autowired
    private SysDictService sysDictService;

    @Around("@annotation(com.mh.user.annotation.TranslationDict)")
    public Object aroundMethodTranslate(ProceedingJoinPoint joinPoint) throws Throwable {
        // 接收到请求时间
        Long startTime = System.currentTimeMillis();
        // 注意，如果调用joinPoint.proceed()方法，则修改的参数值不会生效，必须调用joinPoint.proceed(args)
        Object result = joinPoint.proceed();

        // 获取返回值类型
        Class<?> returnType = ((MethodSignature) joinPoint.getSignature()).getReturnType();

        // 取出要翻译的字典值
        String returnJsonResult = JSONObject.toJSONString(result);

        // 开始解析
        Method method = ((MethodSignature) joinPoint.getSignature()).getMethod();

        // 获取注解上参数
        TranslationDict annotation = method.getAnnotation(TranslationDict.class);
        FieldParam[] fieldParams = annotation.value();

        // 遍历
        for (FieldParam fieldParam : fieldParams) {
            log.info("字典类型：{}，取值字段：{}", fieldParam.type(), fieldParam.targetField());
            Pattern dictPattern = getPattern(fieldParam);
            Matcher matcher = dictPattern.matcher(returnJsonResult);
            StringBuffer sb = new StringBuffer();
            // 转义字段
            this.translateDict(fieldParam, dictPattern, matcher, sb);
            matcher.appendTail(sb);
            returnJsonResult = sb.toString();
        }

        result = JSONObject.parseObject(returnJsonResult, returnType);
        // 如果这里不返回result，则目标对象实际返回值会被置为null
        // 处理完成时间
        Long endTime = System.currentTimeMillis();
        log.info("数据字典转换耗时：{}ms", endTime - startTime);
        return result;
    }

    /**
     * 字典值转义为中文
     *
     * @param fieldParam
     * @param fieldPattern
     * @param fieldMatcher
     * @param sb
     */
    private void translateDict(FieldParam fieldParam, Pattern fieldPattern, Matcher fieldMatcher, StringBuffer sb) {
        //从缓存中一次性取值
        List<SysDict> dictNames = sysDictService.findCacheByType(fieldParam.type());
        while (fieldMatcher.find()) {
            //取出要翻译字段对应的值
            Matcher dictValueMatcher = fieldPattern.matcher(fieldMatcher.group());
            dictValueMatcher.find();
            String group = dictValueMatcher.group();
            //""sex":1", ""sex":"1"",""sex":null"
            //属性无值
            if (group.split(":").length <= 1) continue;
            String dictName = "";
            //获取字典值
            String dictValue = group.split(":")[1].replace("\"", "");
            //属性值非为空 为空赋值空串
            if (StringUtils.isNotBlank(dictValue) && !dictValue.equalsIgnoreCase("null")) {
                //多值
                if (dictValue.split(",").length > 1) {
                    for (String s : dictValue.split(",")) {
                        //fieldParam.dictType() + "_" + s 根据自己字典表设置的规则去查询
                        String type = s;
                        Optional<SysDict> first = dictNames.stream().filter(d -> d.getValue().equals(type)).findFirst();
                        if (!first.isPresent()) continue;
                        dictName += first.get().getLabel() + "/";
                    }
                } else {
                    String type = dictValue;
                    Optional<SysDict> first = dictNames.stream().filter(d -> d.getValue().equals(type)).findFirst();
                    if (!first.isPresent()) continue;
                    dictName = first.get().getLabel();
                }
            }

            String s = "\"" + fieldParam.targetField() + "Str" + "\":\"" + dictName + "\"," + fieldMatcher.group();
            log.info("拼接后字符串:{}", s);
            fieldMatcher.appendReplacement(sb, s);

        }
    }

    private Pattern getPattern(FieldParam fieldParam) {
        Pattern fieldPattern;//属性整型 字符型
        if (fieldParam.targetFieldValueClazz().equals(Integer.class)) {
            fieldPattern = Pattern.compile("\"" + fieldParam.targetField() + "\":(\\d+)?");
        } else {
            fieldPattern = Pattern.compile("\"" + fieldParam.targetField() + "\":\"([0-9a-zA-Z_,]+)?\"");
        }
        return fieldPattern;
    }
}
