package com.mh.user.utils;

import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 缓存工具类
 * @date 2024-11-28 09:42:14
 */
public class CacheUtil {

    private static class Holder {
        private static final CacheUtil INSTANCE = new CacheUtil();
    }

    public static CacheUtil getInstance() {
        return Holder.INSTANCE;
    }

    private final ConcurrentHashMap<String, Cache<Object, Object>> caches = new ConcurrentHashMap<>();
    private final AtomicLong cacheIdCounter = new AtomicLong(0);

    private CacheUtil() {
        // 私有构造函数，防止外部实例化
    }

    /**
     * 创建一个带有指定过期时间的缓存实例
     *
     * @param expireAfterWrite 过期时间（写后）
     * @param unit             时间单位
     * @return 缓存ID
     */
    public void createCache(String key, String value, long expireAfterWrite, TimeUnit unit) {
        Cache<Object, Object> cache = Caffeine.newBuilder()
                .expireAfterWrite(expireAfterWrite, unit)
                .build();
        cache.put(key, value);
        caches.put(key, cache);
    }

    /**
     * 通过缓存ID获取缓存实例
     *
     * @param cacheId 缓存ID
     * @return 缓存实例
     */
    public Cache<Object, Object> getCache(String cacheId) {
        return caches.get(cacheId);
    }

    /**
     * 将键值对放入缓存
     *
     * @param cache 缓存实例
     * @param key   键
     * @param value 值
     */
    public void put(Cache<Object, Object> cache, Object key, Object value) {
        cache.put(key, value);
    }

    /**
     * 从缓存中获取值
     *
     * @param cache 缓存实例
     * @param key   键
     * @return 值
     */
    public Object get(Cache<Object, Object> cache, Object key) {
        return cache.getIfPresent(key);
    }

    /**
     * 从缓存中移除键值对
     *
     * @param cache 缓存实例
     * @param key   键
     */
    public void remove(Cache<Object, Object> cache, Object key) {
        cache.invalidate(key);
    }

}
